import time
import os
from .encoder import Encoder
from .alphabet import Alphabet

# Should change every year with VERSION
# TODO: Automate it
REDUCE_TIME = 1489533827803
VERSION = 3

class ShortId:

    def __init__(self):
        self._counter = 0
        self._previous_delta = 0
        self._encoder = Encoder()
        self._alphabet = Alphabet()

    def generate(self):
        delta = self._get_time_delta()

        if delta == self._previous_delta:
            self._counter += 1
        else:
            self._counter = 0
            self._previous_delta = delta

        res = self._encoder.encode(self._alphabet.lookup, VERSION)
        res = res + self._encoder.encode(self._alphabet.lookup, os.getpid())

        if self._counter > 0:
            res = res + self._encoder.encode(self._alphabet.lookup, self._counter)

        res = res + self._encoder.encode(self._alphabet.lookup, delta)

        return res

    def _get_time(self):
        return int(time.time() * 1000)

    def _get_time_delta(self):
        return int(round((self._get_time() - REDUCE_TIME) * 0.01))



import string
from random import random
from random import shuffle

class Alphabet:

    ORIGINAL = '{0}{1}_-'.format(string.digits, string.ascii_letters)

    def __init__(self):
        self._alphabet = Alphabet.ORIGINAL
        self.reset()

    def set(self, characters):
        characters_len = len(characters)
        original_len = len(Alphabet.ORIGINAL)
        if characters_len != original_len:
            raise RuntimeError('Custom alphabet for shortId must be {0} unique characters. ' +
                               'You submitted {1} characters: {2}'.format(
                                   original_len,
                                   characters_len,
                                   characters
                               ))

        characters_set_len = len(set(characters))
        if characters_set_len != characters_len:
            raise RuntimeError('Custom alphabet for shortId must be {0} unique characters. ' +
                               '{1} non unique values found.'.format(characters_len - characters_set_len))

        self._alphabet = characters
        self.reset()

    def get(self):
        return self._alphabet

    def reset(self):
        self._shuffled = self.shuffle()

    def lookup(self, index):
        return self.get_shuffled()[index]

    def shuffle(self):
        alphabet = list(self._alphabet)
        shuffle(alphabet)
        return ''.join(alphabet)

    def get_shuffled(self):
        return self._shuffled



#!/usr/bin/env python

from distutils.core import setup

__version__ = '0.1.1'

setup(
    name='shortid',
    version=__version__,
    description='Short id generator',
    author='Dmitry Moskowski',
    author_email='me@corpix.ru',
    url='https://github.com/corpix/shortid',
    download_url='https://github.com/corpix/shortid/archive/{0}.tar.gz'.format(__version__),
    keywords = ['short', 'id', 'uuid', 'shortid', 'tinyid'],
    packages=['shortid'],
    license='MIT',
    classifiers=[],
)
